#!/usr/bin/perl -w

package Test::Repocop::TestCache;
use strict;
use DBI;
use Test::Repocop::Metadata;
use Test::Repocop::Options; # just for cachedir, metadata && verbose :(

use File::Path qw(make_path remove_tree);
use File::Basename;

my $repocop_test_outputdirname='testdb';

# static: only to be called Test::Repocop::TestCache::remove_test
sub remove_test {
    return unless @_;
    my @findclause;
    foreach my $testname (@_) {
	push @findclause, '-name',$testname,'-or','-name',$testname.'+++*', '-or';
    }
    pop @findclause if @findclause;
    system ('find',"$repocop_cachedir/$repocop_test_outputdirname",'-type','f','(',@findclause,')','-delete') == 0 || die "error: $!";
}

sub new {
    my $class=shift;
    my $fd;
    my $self = {
	FD => $fd,
	CUR_TEST_FILENAME => '',
	FILTER_NICK=> 0,
    };
    bless $self, $class;
    $self->initialize();
    return $self;
}

sub initialize {
    my $self=shift;
    open ($self->{FD}, " find $repocop_cachedir/$repocop_test_outputdirname/? -type f -name '*+++*' |");
    if (scalar %Repocop::arg::EXCLUDEPACKAGER or scalar %Repocop::arg::INCLUDEPACKAGER) {
	$self->{METADATA}=Test::Repocop::Metadata->new();
	$self->{FILTER_NICK}=1;
    }
}

sub DESTROY {
    my $self=shift;
    close ($self->{FD});
}

sub iterate3 {
    my $self=shift;
    $_=readline $self->{FD};
    return unless $_;
    chomp $_;
    $self->{CUR_TEST_FILENAME}=$_;
    s!^$repocop_cachedir/$repocop_test_outputdirname/./?!!o;
    #print "$_\n";
    my ($rpmkey,$teststatus)=split '/';
    my ($test,$status)=split '\+\+\+', $teststatus;
    return ($rpmkey,$test,$status);
}

sub iterate3_filtered {
    my $self=shift;
    my ($rpm,$test,$status);
    do {
	($rpm,$test,$status)=$self->iterate3();
    } while $rpm && (&Repocop::arg::skip_test($test) || &Repocop::arg::skip_pkg($rpm) || &Repocop::arg::skip_status($status) || ($self->{FILTER_NICK} and &Repocop::arg::skip_nick($self->{METADATA}->nick($rpm))) );
    return unless $rpm;
    return ($rpm,$test,$status);
}


sub iterate4_filtered {
    my $self=shift;
    my ($rpm,$test,$status)=$self->iterate3_filtered();
    return unless $rpm;
    my $result=$self->result();
    return ($rpm,$test,$status,$result);
}

sub result {
    my $self=shift;
    return &_read_file($self->{CUR_TEST_FILENAME});
}


sub _read_file {
    my $filename=shift; 
    return "" unless -s $filename;
    open my $fh, $filename or die $!;
    local $/; # enable localized slurp mode
    my $content = <$fh>;
    close $fh;
    return $content;
}

package Test::Repocop::TestMtimeDB;
use File::Path qw(make_path remove_tree);
use File::Basename;

sub initialize {
    my $repocop_test_outputdir="$Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname";
    make_path($repocop_test_outputdir);
    # for repocop-test-* utils
    $ENV{'REPOCOP_TEST_OUTPUTDIR'}=$repocop_test_outputdir;
}

sub __get_touchfile {
    my ($repocop_pkg_key,$testname)=@_;
    my $prefix=substr($repocop_pkg_key,0,1);
    return "$Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname/$prefix/$repocop_pkg_key/$testname";
}

sub is_test_performed {
    my ($repocop_pkg_key,$testname)=@_;
    return -e &__get_touchfile($repocop_pkg_key,$testname);
}

sub get_test_mtime {
    my ($repocop_pkg_key,$testname)=@_;
    return -M &__get_touchfile($repocop_pkg_key,$testname);
}

sub set_test_mtime {
    my ($repocop_pkg_key,$testname)=@_;
    my $repocop_test_result=&__get_touchfile($repocop_pkg_key,$testname);
    make_path(dirname($repocop_test_result));
    system('touch',$repocop_test_result)==0 or die "touch $repocop_test_result: $!";
}


package Test::Repocop::TestPurge;
use File::Path qw(make_path remove_tree);

# static; called as
# &Test::Repocop::TestPurge::clear_test_result_and_mtime($repocop_pkg_key,$testname);

sub clear_test_result_and_mtime {
    my ($repocop_pkg_key,$testname)=@_;
    my $repocop_test_result=&Test::Repocop::TestMtimeDB::__get_touchfile($repocop_pkg_key,$testname);
    unlink <$repocop_test_result*>;
}

sub new {
    my $class=shift;
    my $fd;
    my $self = {
	FD => $fd,
    };
    bless $self, $class;
    $self->__initialize();
    return $self;
}

sub __initialize {
    my $self=shift;
    open ($self->{FD}, "find $Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname/? -type d |");
}

sub DESTROY {
    my $self=shift;
    close ($self->{FD});
}

sub iterate_pkg {
    my $self=shift;
    while (1) {
	$_=readline $self->{FD};
	return unless $_;
	chomp $_;
	#my $filename=$_;
	s!^$Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname(?:/.)/?!!o;
	next unless $_;
	return $_;
    }
}

sub delete_pkg {
    my ($self,$repocop_pkg_key)=@_;
    return unless $repocop_pkg_key;
    my $prefix=substr($repocop_pkg_key,0,1);
    remove_tree("$Test::Repocop::Options::repocop_cachedir/$repocop_test_outputdirname/$prefix/$repocop_pkg_key");
}

1;

